/*

 SonarEye Geiger Counter Style Ranging
 Copyright © 2012-14 - David Pankhurst
 http://www.utopiamechanicus.com/article/sonar-vision/

*/
#include <avr/pgmspace.h>
// set to -1 to avoid using it and extra power
#define PIN_LED (-1)
// sonar device pins - can be changed
#define PIN_SONAR_VCC     (8)  
#define PIN_SONAR_TRIGGER (PIN_SONAR_VCC-1)  
#define PIN_SONAR_ECHO    (PIN_SONAR_VCC-2)
#define PIN_SONAR_GND     (PIN_SONAR_VCC-3)
//--------------------------------------------------
// audio pin - cannot be changed as tied to interrupt
#define PIN_AUDIO (10)
volatile unsigned char gRate=255;
#define PING_USEC_MAX (20000)
//--------------------------------------------------
void setup()
{
  // for now...
  if (PIN_LED>=0)
    pinMode(PIN_LED, OUTPUT);   
  pinMode(PIN_AUDIO, OUTPUT);    
  pinMode(PIN_SONAR_VCC, OUTPUT);
  pinMode(PIN_SONAR_TRIGGER, OUTPUT);
  pinMode(PIN_SONAR_GND, OUTPUT);
  digitalWrite(PIN_SONAR_VCC, HIGH);
  digitalWrite(PIN_SONAR_GND, LOW);
  cli();
  TCCR1A = 0;
  TCCR1B = 0;
  OCR1A =16384;
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS10);
  TIMSK1 |= (1 << OCIE1A);
  sei();
}
//--------------------------------------------------
void loop()
{
  long value=0, pwr=0;
  while (1)
  {
    if (0<=(value=PingIt()))
    {
      gRate=1+value*254/PING_USEC_MAX;
    }
  }
}
//--------------------------------------------------
ISR(TIMER1_COMPA_vect)
{
  static int waitCount=-2;
  static bool signalOn=false;
  if (--waitCount>=0)
    return;
  if (signalOn)
  {
    digitalWrite(PIN_AUDIO, LOW);
    if (PIN_LED>=0)
      digitalWrite(PIN_LED, LOW);
    waitCount=gRate+10;
  }
  else
  {
    digitalWrite(PIN_AUDIO, HIGH);
    if (PIN_LED>=0)
      digitalWrite(PIN_LED, HIGH);
    waitCount=12;
  }
  signalOn=!signalOn;
}
//--------------------------------------------------
long PingIt()
{
  static unsigned long tickStart=0;
  unsigned long tick=micros();
  if ( tick-tickStart<70000) 
    return -1;
  tickStart=tick;
  digitalWrite(PIN_SONAR_TRIGGER, LOW);
  delayMicroseconds(2);
  digitalWrite(PIN_SONAR_TRIGGER, HIGH);
  delayMicroseconds(10);
  digitalWrite(PIN_SONAR_TRIGGER, LOW);
  delayMicroseconds(2);
  long value=pulseIn(PIN_SONAR_ECHO,HIGH,PING_USEC_MAX);
  if (0==value)
    return PING_USEC_MAX;
  return value;
}

